local L = LibStub('AceLocale-3.0'):GetLocale('TimeToDie', true)

local TimeToDie = LibStub('AceAddon-3.0'):NewAddon('TimeToDie', 'AceEvent-3.0', 'AceConsole-3.0')
_G.TimeToDie = TimeToDie

------------------------------
--      Initialization      --
------------------------------

local defaultFont = GameFontNormal:GetFont()
local events = {target = 'PLAYER_TARGET_CHANGED', focus = 'PLAYER_FOCUS_CHANGED'}
local healing = '|cff00ff00'..SHOW_COMBAT_HEALING ..'|r'

local UnitIsFriend = UnitIsFriend
local UnitHealth = UnitHealth
local GetTime = GetTime
local UnitHealtMax = UnitHealtMax
local format = format

local defaults = {
   profile = {
      frame = true,
      enemy = true,
      target = 'target',
      tot = false,

      locked = false,
      font = defaultFont,
      size = 24,
      r = 1,
      g = .82,
      b = 0,
      outline = '',
      strata = 'LOW',
      justify = 'CENTER',

		p1 = 'CENTER',
		p2 = 'CENTER',
		x = 0,
		y = 0,
   }
}

function TimeToDie:OnInitialize()
   local db = LibStub('AceDB-3.0'):New('TimeToDieDB', defaults, 'Default')
   self.db = db
   local RegisterCallback = db.RegisterCallback
   RegisterCallback(self, 'OnProfileChanged', 'ApplySettings')
   RegisterCallback(self, 'OnProfileCopied', 'ApplySettings')
   RegisterCallback(self, 'OnProfileReset', 'ApplySettings')

   self:RegisterOptions()
end

function TimeToDie:OnEnable()
	self:RegisterEvent('PLAYER_TARGET_CHANGED', 'OnChange')
	self:RegisterEvent('PLAYER_FOCUS_CHANGED', 'OnChange')

   local profile = self.db.profile

   if profile.enemy and profile.tot then
      self:EnableEventFrame()
   else                                -- for profile changed
      self:DisableEventFrame()
   end

   self:RegisterEvent('UNIT_HEALTH')

	if profile.frame then
      if not self.frame then
         self.frame = CreateFrame('Frame', 'TimeToDieFrame', UIParent, 'TimeToDieFrameTemplate')
      end
      self.frame:Show()
      self:ApplySettings()
   elseif self.frame then
      self.frame:Hide()
   end
end

function TimeToDie:OnDisable()
   if self.frame then self.frame:Hide() end
   if self.eventFrame then self:DisableEventFrame() end
end

-------------------------------
--      Event Functions      --
-------------------------------

local health0, time0 -- initial health and time point
local mhealth, mtime -- current midpoint

-- how to check for targettarget or focustarget?
function TimeToDie:UNIT_HEALTH(event, unit)
   local profile = self.db.profile
   local target = profile.target
   if profile.enemy and profile.tot and UnitIsFriend('player', target) then
      target = target..'target'
   end
	if unit ~= target then return end

	local health, time = UnitHealth(target), GetTime()

   if health == UnitHealthMax(target) then
      self.dataobj.text = nil
      health0, time0, mhealth, mtime = nil
      return
	end

	if not health0 then
		health0, time0 = health, time
		return
	end
	if mhealth then
		mhealth, mtime = (mhealth + health) / 2, (mtime + time) / 2
	else
		mhealth, mtime = health, time
	end

	local dhealth = mhealth - health0

   local res
   if dhealth ~= 0 then
		time = health * (time0 - mtime) / dhealth
      if time < 0 then
         res = healing
         health0, time0, mhealth, mtime = nil
		elseif time > 60 then
			res = format('%d:%02d', time/60, time%60)
		else
			res = format('%d', time%60)
		end
	else
		res = '0s'
	end
   self.dataobj.text = res
end

function TimeToDie:OnChange(event)
   local profile = self.db.profile
   local target = profile.target
	if events[target] ~= event then return end
	self.dataobj.text, health0, time0, mhealth, mtime = nil
end

local oldhealth = nil
local function OnUpdate(self, elapsed)
   local t = self.elapsed + elapsed
   if t < .1 then
      self.elapsed = t
      return
   end
   self.elapsed = 0

   --frame's onupdate calls UNIT_HEALTH with targettarget or focustarget
   local unit = self.unit
   local health = UnitHealth(unit)
   if oldhealth ~= health then
      oldhealth = health
      TimeToDie:UNIT_HEALTH('UNIT_HEALTH', unit)
   end
end

local function OnEvent(self, event, unit) -- event == UNIT_TARGET
   local profile = TimeToDie.db.profile
   if unit == profile.target and UnitIsFriend('player', unit) then
      TimeToDie:OnChange(events[unit])
      unit = unit..'target'
      if UnitExists(unit) then
         frame:SetScript('OnUpdate', OnUpdate)
         self.unit = unit
      else
         frame:SetScript('OnUpdate', nil)
      end
   end
end

-- profile.enemey and profile.tot = true
-- only use if target is friendly
function TimeToDie:EnableEventFrame()
   local frame = self.eventFrame or CreateFrame('Frame')
   self.eventFrame = frame
   frame:RegisterEvent('UNIT_TARGET')  -- call OnChange?
   frame:SetScript('OnEvent', OnEvent)
   frame.elapsed = 0

--~    self:UnregisterEvent('UNIT_HEALTH')
end

function TimeToDie:DisableEventFrame()
   local frame = self.eventFrame
   if not frame then return end
   frame:SetScript('OnUpdate', nil)
   frame:UnregisterAllEvents()
--~    self:RegisterEvent('UNIT_HEALTH')
end

function TimeToDie:ApplySettings(profile)
   local frame = self.frame
   if not frame then return end
   local bg, text = frame.bg, frame.text
   local profile = self.db.profile
	frame:SetFrameStrata(profile.strata)
   frame.text:SetJustifyH(profile.justify)
	frame:ClearAllPoints()
	frame:SetPoint(profile.p1, UIParent, profile.p2, profile.x, profile.y)
	if profile.locked then
		bg:Hide()
		frame:EnableMouse(false)
      self.dataobj.text = nil
	else
		bg:Show()
		frame:EnableMouse(true)
      self.dataobj.text = 'TimeToDie'
	end
	text:SetFont(SML and SML:Fetch('font', profile.font) or defaultFont, profile.size, profile.outline)
	text:SetTextColor(profile.r,profile.g,profile.b)
	timer, health0, time0, mhealth, mtime = nil
end

TimeToDie.dataobj = LibStub:GetLibrary('LibDataBroker-1.1'):NewDataObject('TimeToDie', {
   type = 'data source',
   text = 'TTD',
   label = 'TTD',
   unit = 'time',
   icon = 'Interface\\Icons\\Spell_Holy_BorrowedTime',
})
